package per.logic;

import java.io.Serializable;
import java.util.ArrayList;

import per.db.DbConnector;
import per.location.Locator;
import per.mail.MailClient;
import per.model.Friend;
import per.model.FriendLocation;
import per.model.FriendStatus;
import per.model.Message;
import per.model.User;
import pos.Position;

public class ProgramController implements Serializable{
	/**
	 * 
	 */
	private static final long serialVersionUID = 0L;
	private DbConnector dbc; 
	private User loggedInUser;
	private Locator locator;
	private ArrayList<User> allUsers;
	
	
	
	public ProgramController(String nickname, String password){
		dbc = new DbConnector();
		
		loggedInUser = dbc.getUser(nickname, password);
		if (loggedInUser!=null){
			locator = new Locator(dbc, loggedInUser);
			loggedInUser.setPosition(locator.getOwnPositionStartThread());
			loggedInUser.setFriends(dbc.getAllFriendsPosition(loggedInUser.getFriends()));
		}
		
	}
	
	public ProgramController(){
		dbc = new DbConnector();
	}
	

	public ArrayList<User> getAllUsers() {
		return dbc.getAllUsers(loggedInUser.getUserId());
	}
	public void setAllUsers(ArrayList<User> allUsers) {
		this.allUsers = allUsers;
	}
	public void setPosition(){
		loggedInUser.setPosition(locator.getPositionWithoutSave());
	}
	public void updatePosition(){
		loggedInUser.setPosition(locator.getOwnPosition());
	}
	public User getLoggedInUser(){
		return loggedInUser;
	}
	public void setLoggedInUser(User u){
		this.loggedInUser=u;
	}
	public void updateRecivedMessages(){
		loggedInUser.setRecievedMessages(dbc.getRecievedMessages(loggedInUser));
	}
	public void newUser(User u){
		dbc.insertUser(u);
	}
	public void updateUser(User u) throws InterruptedException{
		
		dbc.updateUser(u);
		
	}
	public void updateUser(){
		loggedInUser = dbc.getUser(loggedInUser.getUserId(), true, true);
	}
	public void sendMessage(Message m){
		m.setSender(loggedInUser);
		dbc.insertMessage(m);
		
	}
	
	public void deleteMessage(Message m){
		
	}
	public void updateAllFriendsPosition(){
		
		loggedInUser.setFriends(dbc.getAllFriendsPosition(loggedInUser.getFriends()));
		
	}
	public User getFriend(String userId){
		int id = Integer.parseInt(userId);
		for (Friend f: loggedInUser.getFriends()){
			if (f.getFriend().getUserId()==id){
				User fr = f.getFriend();
				
				return fr;
			}
		}
		return null;
		
	}
	
	public void updateFriendList(){
		loggedInUser.setFriends(dbc.getFriendList(loggedInUser));
	}
	
	public void addFriend(int friend, String status){
		dbc.insertFriend(loggedInUser.getUserId(), friend, status);
	}
	
	public String getDistanceColor(Position p){
		if ((p.getLatitude()==0.0 && p.getLongitude() == 0.0) || (loggedInUser.getPosition().getLatitude()==0.0 && loggedInUser.getPosition().getLongitude() == 0.0)){
			return "red";
		}
		if ((Math.abs(p.getLatitude()-loggedInUser.getPosition().getLatitude())<0.003)&&(Math.abs(p.getLongitude()-loggedInUser.getPosition().getLongitude())<0.003)){
			return "green";
		}
		else{
			return "yellow";
		}
		
	}
	public void setMessageRead(String messageId){
		dbc.setMessageRead(Integer.parseInt(messageId));
	}
	public void insertUser(User u){
		dbc.insertUser(u);
	}
	public boolean isNicknameTaken(String nickname){
		if(loggedInUser != null){
			if (loggedInUser.getNickname().equals(nickname)){
				return false;
			}
		}
		return dbc.nicknameTaken(nickname);
	}
	
	public User getUser(String userId){
		int id = Integer.parseInt(userId);
		return dbc.getUser(id, true, true);
	}
	public void changeFriendStatus(int friend, String status){
		dbc.updateFriendStatus(loggedInUser.getUserId(), friend, status);
	}
	
	public int editPass(String newPass, String oldPass){
		
		return dbc.updatePassword(newPass, oldPass, loggedInUser.getNickname());
	}
	public String sendNewPassword(String mail){
		int id= dbc.getUserByMail(mail);
		if (id==0){
			return "Mail-adressen "+ mail +" er ikke registrert i systemet.";
		}
		else{
			MailClient mc = new MailClient();
			try{
				String pass = mc.sendMail(mail);
				System.out.println("id:" + id+ ", pass: "+pass);
				dbc.updatePassword(id, pass);
				return "Nytt passord er sendt til "+mail;
			}catch(Exception e){
				e.printStackTrace();
				return "Noe gikk feil. Prv igjen.";
			}
			
			
		}
		
		
	}
	
	

}
